pro L1B2_to_ASCII
;
; This program reads the data for each of the four bands in a MISR Level 1B2
; file and writes the data into ASCII files, with one file for each block
; in each band.
;
; Input:
; There are no input parameters to the routine.  The user is prompted for the name
; of a "listfile" using a typical file selection GUI.  The  listfile should contain
; the file names (with complete directory path) of the MISR Level 1B2 files to be
; converted to ASCII, one file name per line.  By default, the dialog for specifying
; this file begins by looking in the directory where this program resides.
;
; Output:
; One file is written for each block containing good (non-fill) data** for each
; parameter (band), in ASCII text format. The files are written to the directory
; that contains the MISR Level 1B2 file being read, and the files are named
; using the original file name with the band and block number appended and
; a .txt suffix, e.g.
;
;     MISR_AM1_GRP_ELLIPSOID_GM_P016_O029945_AF_F03_0024_BlueBand_Block19.txt

; A MISR block is dimensioned with m samples or pixels by n lines, e.g. 512 samples by
; 128 lines or 2048 samples by 512 lines depending on the camera and band, and that
; is the format used in the output file.  That is, there are 128 lines of data, each
; containing 512 values or 512 lines of data, each containing 2048 values.
;
; ** MISR makes measurements in the daylit portion of each orbit.  There are 180
; blocks defined to cover full extent of the daylit orbit range, but only about 144
; of those 180 blocks fall within the daylit portion for any given orbit.  The range
; of blocks containing good data for a particular orbit varies depending on the time
; of year.  The remaining blocks contain only fill values and are not written out
; by this routine.

listfile = DIALOG_PICKFILE(TITLE="Select the list file containing MISR L1B2file paths")
misrfiles = STRARR(FILE_LINES(listfile))
OPENR,ilun,listfile,/GET_LUN
READF,ilun,misrfiles
FREE_LUN,ilun

for i=0,N_ELEMENTS(misrfiles)-1 do begin
  mfile = misrfiles[i]
  mdirname = FILE_DIRNAME(mfile)
  mfilename = FILE_BASENAME(mfile,'.hdf')

  num_grid = EOS_GD_INQGRID(mfile,gridlist)
  grids = STRSPLIT(gridlist,',',/EXTRACT)

  ; get start and end blocks for good data range for the file

  status = read_global_attr(mfile,"Start_block",stbl)
  status = read_global_attr(mfile,"End block",endbl)
  startblock = stbl[0]
  endblock = endbl[0]

  for j=0,3 do begin ; loop through the first 4 grids, which are the bands
    ; get the band data field name
    fid = EOS_GD_OPEN(mfile,/READ)
    gid = EOS_GD_ATTACH(fid,grids[j])
    numfields = EOS_GD_INQFIELDS(gid,fieldlist,rank,numtype)
    status = EOS_GD_DETACH(gid)
    status = EOS_GD_CLOSE(fid)

    ; get the name of the radiance field and build the output
    ; format statement based on the block dimensions for that field
    fields = STRSPLIT(fieldlist,',',/EXTRACT)
    status = read_grid_field(mfile,grids[j],fields[0],data)
    sz = size(data)
    xsize = strtrim(string(sz[1]),2)
    ysize = strtrim(string(sz[2]),2)
    fmt = "("+ysize+"("+xsize+"(I5,1X)/))"

    ; for each block in the good data range, write the parameter data for that block
    for k=startblock-1,endblock-1 do begin
      ofile = mdirname + path_sep() + mfilename + $
              "_"+STRTRIM(grids[j])+"_Block"+STRTRIM(STRING(k+1),2)+".txt"
      OPENW,olun,ofile,/GET_LUN
      PRINTF,olun,data[*,*,k],FORMAT=fmt
      FREE_LUN,olun
    endfor ; loop on blocks
  endfor; loop on grids

endfor; loop on files

result = DIALOG_MESSAGE("Processing is complete",/INFORMATION)

end
